//***********************************************************************//
//		$Author:		DigiBen		digiben@gametutorials.com			 //
//		$Program:		3DS Loader										 //
//		$Description:	Demonstrates how to load a .3ds file format		 //
//																		 //
//		$Modified by:	Marcio Serolli Pinho	pinho@inf.pucrs.br		 //
//						May, 17th 2002									 //
//						To be used with GLUT						     //
//***********************************************************************//

#include <windows.h>

#include "3ds.h"										// Include the 3DS header file.

t3DModel Objeto;
t3DModel Objeto2;

float g_RotateX		  = 0.0f;							// This is the current value at which the model is rotated
float g_RotationSpeed = 0.8f;							// This is the speed that our model rotates.  (-speed rotates left)

// **********************************************************************
//  void DefineLuz(void)
//
//
// **********************************************************************
void DefineLuz(void)
{
  // Define cores para um objeto dourado
  GLfloat LuzAmbiente[]   = {0.24725f, 0.1995f, 0.07f } ;
  GLfloat LuzDifusa[]   = {0.75164f, 0.60648f, 0.22648f, 1.0f };
  GLfloat LuzEspecular[] = {0.626281f, 0.555802f, 0.366065f, 1.0f };
  GLfloat PosicaoLuz0[]  = {3.0f, 3.0f, 0.0f, 1.0f };
  GLfloat PosicaoLuz1[]  = {-3.0f, -3.0f, 0.0f, 1.0f };
  GLfloat Especularidade[] = {1.0f, 1.0f, 1.0f, 1.0f };

   // ****************  Fonte de Luz 0

	glEnable ( GL_COLOR_MATERIAL );


   // Habilita o uso de iluminao
  glEnable(GL_LIGHTING);

  // Ativa o uso da luz ambiente
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
  // Define os parametros da Luz nmero Zero
  glLightfv(GL_LIGHT0, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT0, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT0, GL_POSITION, PosicaoLuz0 );
  glEnable(GL_LIGHT0);

  // Ativa o "Color Tracking"
  glEnable(GL_COLOR_MATERIAL);

  // Define a reflectancia do material
  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  // Define a concentrao do brilho.
  // Quanto maior o valor do Segundo parametro, mais
  // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
  glMateriali(GL_FRONT,GL_SHININESS,51);

  // ****************  Fonte de Luz 1

  // Ativa o uso da luz ambiente
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
  // Define os parametros da Luz nmero Zero
  glLightfv(GL_LIGHT1, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT1, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT1, GL_POSITION, PosicaoLuz1 );
  glEnable(GL_LIGHT1);

  // Ativa o "Color Tracking"
  glEnable(GL_COLOR_MATERIAL);

  // Define a reflectancia do material
  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  // Define a concentrao do brilho.
  // Quanto maior o valor do Segundo parametro, mais
  // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
  glMateriali(GL_FRONT,GL_SHININESS,20);

}


void RenderScene() 
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	
	DefineLuz();

	glLoadIdentity();									// Reset The matrix

	gluLookAt(0, 1.5f, 50.0,0, 0.5f, 0,0, 1, 0);
	
	// We want the model to rotate around the axis so we give it a rotation
	// value, then increase/decrease it. You can rotate right of left with the arrow keys.

	glPushMatrix();
		glTranslatef(1.5f,6.0f,0);
		glScalef(4,4,4);
		glRotatef(g_RotateX, 0, 1.0f, 0);						// Rotate the object around the Y-Axis
		g_RotateX += g_RotationSpeed;							// Increase the speed of rotation
		Draw3DSObject (&Objeto);
	glPopMatrix();

	glPushMatrix();
		glScalef(0.1f, 0.1f, 0.1f);
		glTranslatef(-1.5,0,0);
		glRotatef(-g_RotateX, 0, 1.0f, 0);						// Rotate the object around the Y-Axis
		g_RotateX += g_RotationSpeed;							// Increase the speed of rotation
		Draw3DSObject (&Objeto2);
	glPopMatrix();

	glutSwapBuffers();
}

// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )
//
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )  
{
	switch ( key ) 
	{
    case 27:      
		
		FreeObjectData(&Objeto);
		FreeObjectData(&Objeto2);
					// Termina o programa qdo
      exit ( 0 );   // a tecla ESC for pressionada
      break;        
    default:        
      break;
  }
}
// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void init(void)
{
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Fundo de tela preto

	glShadeModel(GL_SMOOTH);						
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_DEPTH_TEST);
	
	glEnable ( GL_CULL_FACE );

	glEnable(GL_LIGHT0);								
	glEnable(GL_LIGHTING);							
	glEnable(GL_COLOR_MATERIAL);	
	glDisable(GL_CULL_FACE);
}

// **********************************************************************
//  void reshape( int w, int h )
//		trata o redimensionamento da janela OpenGL
//
// **********************************************************************
void reshape( int width, int height )
{
	if (height==0)										// Prevent A Divide By Zero error
	{
		height=1;										// Make the Height Equal One
	}

	glViewport(0,0,width,height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();			

	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, .5f ,150.0f);

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();
}



// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
void main ( int argc, char** argv )   
{

	glutInit            ( &argc, argv ); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);// | GLUT_DOUBLE | GLUT_RGBA ); 
	glutInitWindowPosition (0,0);
	glutInitWindowSize  ( 700, 500 ); 
	glutCreateWindow    ( "Displaying 3DS files with GLUT." ); 
		
	init ();

//	Load3DSObject (&Objeto, "face.3ds");
	Load3DSObject (&Objeto2, "bed29.3ds");	
	Load3DSObject (&Objeto2, "Banana.3ds");	


	glutDisplayFunc ( RenderScene );  
	glutReshapeFunc ( reshape );
	glutKeyboardFunc ( keyboard );
	glutIdleFunc ( RenderScene );

	glutMainLoop ( );          
	
}

